import string
from functools import wraps


class dummy_lru_cache:
    def __init__(self, *args, **kwargs):
        pass

    def __call__(self, f):
        @wraps(f)
        def wrapped(*args, **kwargs):
            return f(*args, **kwargs)
        return wrapped

def process(previous, current, next_chr):
    uppercase = string.ascii_uppercase
    if not previous and current in uppercase:
        return current.lower()
    if previous and current in uppercase and previous in uppercase:
        return current.lower()
    if current in uppercase:
        return ["_", current.lower()]
    return current

def uncamelize(word):
    result = list()

    for index, char in enumerate(word):
        previous = word[index - 1] if index > 0 else None
        next_chr = word[index + 1] if index < len(word) - 1 else None
        result.append("".join(process(previous, char, next_chr)))
    return "".join(result)



try:
    from functools import lru_cache
except ImportError:
    from rancher.utils import dummy_lru_cache as lru_cache

from rancher.models import RancherEnvironment, RancherApiHome, RancherAccount
from rancher.errors import ApiException, ExistentObjectException
from rancher.engine import RequestAdapter


class ApiSettings:
    url = None
    access_key = None
    secret_key = None
    account = None


class RancherApi:

    def __init__(self, settings, request_adapter):
        self.settings = settings
        self.http = request_adapter

    @classmethod
    def client(cls, url):
        settings = ApiSettings()
        settings.url = url
        http_client = RequestAdapter()
        return cls(settings, http_client)

    @property
    @lru_cache(maxsize=10)
    def links(self):
        response = self.http.get(self.settings.url)
        return RancherApiHome.from_dict(response.json()).links

    @property
    @lru_cache(maxsize=100)
    def accounts(self):
        response = self.http.get(self.links.accounts)
        if not response.ok:
            raise ApiException(response.json()['message'])
        return [RancherAccount.from_dict(i) for i in response.json()['data']]

    @property
    @lru_cache(maxsize=100)
    def envs(self):
        envs = self.http.get(self.settings.url + '/environments').json()['data']
        return [RancherEnvironment.from_dict(i) for i in envs]

    @property
    def projects(self):
        return [i for i in self.accounts if i.type == 'project']


    def get_env(self, name=None):
        for env in self.envs:
            if env.name.lower() == name.lower():
                return env

    def get_project(self, name=None):
        """
        Search a project by name in a case-insensitive form.
        """
        for project in self.projects:
            if project.name.lower() == name.lower():
                return project

    def get_account(self, name=None):
        """
        Search an account by name in a case-insensitive form.
        """
        for account in self.accounts:
            if account.name.lower() == name.lower():
                return account
        return None

    def create_env(self, name, account, description="", docker_compose="", rancher_compose=""):
        if not isinstance(account, RancherAccount):
            raise TypeError(
                'Provide a RancherAccount object'
                ' for the parameter account'
            )

        if not account.type == 'project':
            raise TypeError(
                "An account with type 'project' "
                "has to be given. Provided one was"
                " of type: {}"
                .format(account.type)
            )

        data = {
            'name': name,
            'description': description,
            'dockerCompose': docker_compose,
            'rancherCompose': rancher_compose,
            'startOnCreate': True,
        }
        response = self.http.post(
            self.settings.url + "/projects/{}/environment".format(account.id),
            json=data
        )
        if not response.ok:
            if response.status_code == 422:
                raise ExistentObjectException("The environment already exists")
            message = response.json()['message']
            raise ApiException(message)

        return RancherEnvironment.from_dict(response.json())

    def remove_env(self, environment):
        environment.remove()



import inspect
import types
from abc import ABCMeta, abstractmethod

from rancher.utils import uncamelize

import requests


class JsonMarshable:

    uncamelize = True

    @classmethod
    def get_members(cls):
        members = inspect.getmembers(cls)
        members = filter(lambda e: not e[0].startswith("__"), members)
        members = filter(lambda e: not isinstance(e[1], types.MethodType), members)
        members = filter(lambda e: not (callable(e[1]) and not inspect.isclass(e[1])), members)
        return {name: value for name, value in members}

    @classmethod
    def from_dict(cls, dict_repr):
        members = cls.get_members()
        instance = cls()
        dict_repr = instance.uncamelize_keys(dict_repr) if cls.uncamelize else dict_repr
        for key, value in dict_repr.items():
            if key in members.keys():
                if members[key] and issubclass(members[key], Model):
                    setattr(instance, key, getattr(cls, key).from_dict(value))
                    continue
                setattr(instance, key, value)
        instance._rawdata = dict_repr
        return instance

    def uncamelize_keys(self, representation):
        result = dict()
        if not representation:
            return dict()
        for key, value in representation.items():
            if isinstance(value, dict):
                value = self.uncamelize_keys(value)
            result.update({uncamelize(key): value})
        return result

    def to_dict(self):
        obj = dict()
        for field, value in self.get_members().items():
            if field in JsonMarshable.get_members():
                continue
            value_instance = getattr(self, field)

            if value_instance and inspect.isclass(value) and issubclass(value, Model):
                obj[field] = value_instance.to_dict()
            else:
                obj[field] = value_instance

        return obj


class Model:

    def __init__(self, **kwargs):
        class_members = inspect.getmembers(self.__class__)
        class_members = dict(filter(lambda e: not e[0].startswith("__"), class_members))
        is_model_class = lambda e: inspect.isclass(e) and issubclass(e, Model)
        for name, value in kwargs.items():
            # If the atribute is defined in the model as a nested model then check
            # if the object given is an instance of that class.
            if is_model_class(class_members[name]) and not isinstance(value, Model):
                raise ValueError(
                    "Attribute '{}' is defined as {} type in {}. '{}' instance was given instead."
                    .format(
                        name,
                        class_members[name].__name__,
                        self.__class__.__name__,
                        value.__class__.__name__)
                )
            setattr(self, name, value)
        # Search for nested uninitialized models and set them to None.
        for name, member in inspect.getmembers(self):
            if is_model_class(member) and not name.startswith("__"):
                setattr(self, name, None)
        #TODO: USE DEPENDENCY INJECTION FOR THE MOTHER OF GOD
        setattr(self, '_http', RequestAdapter())

    def __repr__(self):
        if hasattr(self, 'name'):
            return "<{} {}>".format(
                self.__class__.__name__,
                getattr(self, 'name')
            )
        else:
            return super().__repr__()


class HttpInterface():
    __metaclass__ = ABCMeta

    @abstractmethod
    def get(self, url):
        pass

    @abstractmethod
    def post(self, url, *extra, **kwargs):
        pass

    @abstractmethod
    def delete(self, url):
        pass

    @abstractmethod
    def put(self, url):
        pass


class RequestAdapter(HttpInterface):

    def __init__(self):
        self.session = requests.Session()

    def get(self, url):
        return requests.get(url)

    def post(self, url, *extra, **kwargs):
        return requests.post(url, **kwargs)

    def delete(self, url):
        pass

    def put(self, url):
        pass



from setuptools import setup


setup(
    name="rancher",
    version="0.3.5",
    author="Miguel Ángel Durán González",
    author_email="hi@mangel.me",
    description="Python wrapper for rancher api",
    url="https://github.com/mangeld/rancher_api",
    packages=['rancher',],
    install_requires=[
        'requests==2.12.1',
        'websocket-client==0.40.0',
    ],
)
